(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_path.mli,v 1.4 2014/01/31 09:51:52 fclement Exp $ *)

(** {3 Defining a path machinery to find files in the file system} *)

exception Empty;;
(** This exception is raised when [pop] is called and the current
   directory path is empty. *)

type t = Og_file.dir_name list;;
(** A directory path is a list of directory names. *)

(** {6 The path machinery:
   define a current list of directories to search files into} *)
val get : unit -> t;;
val push : Og_file.dir_name -> unit;;
(** Raises [Failure] if the directory is not readable or non existent. *)
val pop : unit -> unit;;
val init : t -> unit;;

val find : Og_file.file_name -> Og_file.explicit_file_name;;
(** Return the explicit file name of the first file with the given name
    in the current directory path. *)

val print : Format.formatter -> t -> unit;;
(** Print the given [Path.t] into the [Formatter.formatter]. *)

(** {6 In path channel manipulation} *)
val open_in : Og_file.file_name -> in_channel;;
val open_in_bin : Og_file.file_name -> in_channel;;
val open_out : Og_file.file_name -> out_channel;;
val open_out_bin : Og_file.file_name -> out_channel;;
(** Opening files in the current path. Equivalent to the regular open
   primitives if the file name argument is not implicit. *)
