(* $Id: file_arg.ml,v 1.1.1.1 2012/05/21 21:32:14 fclement Exp $ *)

type ic = In of string option;;
type oc = Out of string option;;

let source = ref (In None);;
let target = ref (Out None);;

let args = ref [];;

let anonymous s =
  if !source = In None then source := In (Some s) else target := Out (Some s);
  if !target <> Out None then begin
    args := (!source, !target) :: !args;
    source := In None;
    target := Out None
  end
;;

let ofile s =
  match !args with
  | [] -> args := (In None, Out (Some s)) :: !args
  | (s1, Out None) :: rest -> args := (s1, Out (Some s)) :: rest
  | (_, Out (Some _)) :: _ -> args := (In None, Out (Some s)) :: !args
;;

let ifile s =
  match !args with
  | [] -> args := (In (Some s), Out None) :: !args
  | (In None, s2) :: rest -> args := (In (Some s), s2) :: rest
  | (In (Some _), _) :: _ -> args := (In (Some s), Out None) :: !args
;;

let list () =
  if !args = [] then [!source, !target] else List.rev !args
;;

let open_in = function
  | In None -> stdin
  | In (Some s) -> Pervasives.open_in s
;;

let open_out = function
  | Out None -> stdout
  | Out (Some s) -> Pervasives.open_out s
;;

let close_in ic = if ic != stdin then Pervasives.close_in ic;;
let close_out oc =
  if oc == stdout then Pervasives.flush oc else Pervasives.close_out oc
;;

let clean_file = function
  | Out None -> ()
  | Out (Some s) ->
    try Sys.remove s with
    | Sys_error _ -> ()
;;

let oc_to_string = function
  | Out None -> "stdout"
  | Out (Some s) -> s
;;

let ic_to_string = function
  | In None -> "stdin"
  | In (Some s) -> s
;;
