(* $Id: pr_type.ml,v 1.2 2012/05/22 15:11:17 fclement Exp $ *)

(* pr_type.ml: impression des types dclars dans le fichier de
   description de la base de donnes. *)

open Buffer;;
open Types;;

let print_types = ref false;;

let type_definitions_are_printed () = !print_types;;
let keep_type_definitions () = print_types := true;;

let var_type b v = bprintf b "'%s" v;;

let rec type_expr0 b = function
  | Var_type s -> bprintf b "%a" (var_type) s
  | Const_type ty_name -> type_longident b ty_name
  | ty -> bprintf b "(%a)" (type_expr) ty

and type_application b = function
  | Fun_type (ty_name, ty_v) ->
    begin match Array.length ty_v with
    | 0 -> ()
    | 1 -> type_application b ty_v.(0)
    | n ->
      bprintf b "(%a" (type_application) ty_v.(0);
      for i = 1 to n - 1 do bprintf b ", %a" (type_application) ty_v.(i) done;
      bprintf b ")"
    end;
    bprintf b " %a" (type_longident) ty_name
  | ty -> type_expr0 b ty

and type_tuples b = function
  | Tuple_type tys -> bprintf b "%a" (tuple_fields) tys
  | ty -> type_application b ty

and tuple_fields b fields =
  bprintf b "%a" (type_application) fields.(0);
  for i = 1 to Array.length fields - 1 do
    bprintf b " * %a" (type_application) fields.(i)
  done

and type_expr b = type_tuples b

and type_longident b = function
  | Lident id -> bprintf b "%s" id
  | Ldot (longident, id) -> bprintf b "%a.%s" (type_longident) longident id
  | Lapply (foncident, modident) ->
    bprintf b "%a(%a)" (type_longident) foncident (type_longident) modident
;;

let record_field b = function
  | Field (lab, ty_desc) ->
    bprintf b "%s : %a" lab (type_expr) ty_desc
  | Field_mutable (lab, ty_desc) ->
    bprintf b "mutable %s : %a" lab (type_expr) ty_desc
;;

let rec record_fields b = function
  | [] -> ()
  | rf :: rfs ->
    bprintf b " %a;\n%a" (record_field) rf (record_fields) rfs
;;

let sum_field b = function
  | Sum_tag lab ->
    bprintf b "%s" lab
  | Sum_inj (lab, ty_desc) ->
    bprintf b "%s of %a" lab (type_expr) ty_desc
;;

let rec sum_fields b = function
  | [] -> ()
  | [sf] ->
    bprintf b "| %a" (sum_field) sf
  | sf :: sfs ->
    bprintf b "| %a\n%a" (sum_field) sf (sum_fields) sfs
;;

let type_kind b = function
  | Record_type rfs ->
    bprintf b "{\n%a}\n" (record_fields) rfs
  | Sum_type sfs ->
    bprintf b "\n%a\n" (sum_fields) sfs
  | Expr_type ty_desc ->
    bprintf b "\n %a\n" (type_expr) ty_desc
;;

let type_arguments b = function
  | [] -> ()
  | [v] -> bprintf b "'%s " v
  | v0 :: l ->
    let ty_args b l = List.iter (fun v -> bprintf b ", '%s" v) l in
    bprintf b "('%s%a) " v0 (ty_args) l
;;

let type_name b s = bprintf b "%s" s;;

let type_def b (ty_name, ty_args, ty_kind) =
  bprintf b "%a%a = %a"
    (type_arguments) ty_args (type_name) ty_name (type_kind) ty_kind
;;

let rec type_defs b = function
  | [] -> ()
  | [ty_def] ->
    bprintf b "%a" (type_def) ty_def
  | ty_def :: ty_defs ->
    bprintf b "%a\nand %a" (type_def) ty_def (type_defs) ty_defs
;;

let comment s = s;;

let phrase b = function
  | Comment s -> bprintf b "%s" (comment s)
  | Directive s -> bprintf b "open %s;;\n" s
  | Db_name (None) -> ()
  | Db_name (Some dbn) ->
    bprintf b "let base_name = \"%s\";;\n\n" dbn
  | Type_def ty_defs ->
    bprintf b "type %a;;\n" (type_defs) ty_defs
;;

let rec phrases b = function
  | [] -> ()
  | ph :: phs -> bprintf b "%a\n%a" (phrase) ph (phrases) phs
;;

let adb b phs =
  if !print_types then
    bprintf b "%a\n" (phrases) phs
;;

let print_type_expr = type_expr;;
let print_type_name = type_name;;
